import java.sql.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.Calendar;

public class Database {
    String schedule[];
    public Connection connect() {
        Connection conn = null;
        try {
            //Class.forName("org.sqlite.JDBC");
            String url = "jdbc:sqlite:testdatabase.db";
            conn = DriverManager.getConnection(url);
            System.out.println("A new database connection has been made.");
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        return conn;
    }

    //Adds user to database, returns false if user already exists.
    public boolean addUser (String name, String passw,  String role, String email) {
        String sqlUsers = "INSERT INTO users(name,passw,role,email) VALUES(?,?,?,?)";
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sqlUsers)) {
            if (existingUser(name) == 0) {
                pstmt.setString(1, name);
                pstmt.setString(2, passw);
                pstmt.setString(3, role);
                pstmt.setString(4, email);
                pstmt.executeUpdate();
                System.out.println("New user " + name + " added");
                return true;
            }
        } catch (SQLException e) {
            System.out.println(e.getMessage());
        }
        System.out.println("Failed to add new user " + name);
        return false;
    }

    //Checks if the password matches the user
    public boolean checkCredential (String name, String passw) {
        String sql = "SELECT passw FROM users WHERE name =?";
        System.out.println("Checking credentials for " + name);
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sql)) {
            pstmt.setString(1, name);
            ResultSet rs  = pstmt.executeQuery();
            while (rs.next()) {
                String dbPass = rs.getString("passw");
                System.out.println(dbPass);
                if (dbPass.equals(passw)) {
                    System.out.println("Login succeeded");
                    return true;
                }
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        System.out.println("Login failed");
        return false;
    }

    //Checks if a user with said name exists and returns their ID
    public int existingUser (String name) {
        String sql = "SELECT id FROM users WHERE name =?";
        System.out.println("Checking if user " + name + " exists");
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sql)) {
            pstmt.setString(1, name);
            ResultSet rs  = pstmt.executeQuery();
            while (rs.next()) {
                int dbID = rs.getInt("id");
                System.out.println("User " + name + " exists with id " + dbID);
                return dbID;
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        System.out.println("User " + name + " does not exist");
        return 0;
    }


    //Changes privacy


     //Checks user role
    public String checkRole (String name) {
        System.out.println("Checking role  of " + name);
        String sql = "SELECT role FROM users WHERE name =?";
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sql)) {
            pstmt.setString(1, name);
            ResultSet rs = pstmt.executeQuery();
            while (rs.next()) {
                String dbRole = rs.getString("role");
                return dbRole;
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        return "";
    }

    public String checkRole (int id) {
        String dbRole = "";
        String sql = "SELECT role FROM users WHERE id =?";
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sql)) {
            pstmt.setInt(1, id);
            ResultSet rs  = pstmt.executeQuery();
            while (rs.next()) {
                dbRole = rs.getString("role");

            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        return dbRole;
    }

    public String checkPrivacy (int id) {
        String dbPrivacy = "";
        String sql = "SELECT privacy FROM users WHERE id =?";
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sql)) {
            pstmt.setInt(1, id);
            ResultSet rs  = pstmt.executeQuery();
            while (rs.next()) {
                dbPrivacy = rs.getString("privacy");

            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        return dbPrivacy;
    }

    public void changePrivacy (int id, String s) {
        String sql = "UPDATE users SET privacy = ? WHERE id = ?";
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sql)) {
            pstmt.setString(1, s);
            pstmt.setInt(2, id);
            pstmt.executeUpdate();
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
    }

    public void changeReady (int id, boolean value) {
        String sql = "UPDATE users SET ready = ? WHERE id = ?";
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sql)) {
            pstmt.setBoolean(1, value);
            pstmt.setInt(2, id);
            pstmt.executeUpdate();
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
    }

    public boolean checkReady (int id) {
        boolean dbReady = false;
        String sql = "SELECT ready FROM users WHERE id =?";
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sql)) {
            pstmt.setInt(1, id);
            ResultSet rs  = pstmt.executeQuery();
            while (rs.next()) {
                dbReady = rs.getBoolean("ready");

            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        return dbReady;
    }

    public int checkOther (int id) {
        int dbOther = 0;
        String sql = "SELECT partner FROM users WHERE id =?";
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sql)) {
            pstmt.setInt(1, id);
            ResultSet rs  = pstmt.executeQuery();
            while (rs.next()) {
                dbOther = rs.getInt("partner");

            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        return dbOther;
    }

    public void findEarliest (int id) {
        //finds earliest time that matches both users
        String sqlUsers =
                "SELECT date FROM appointment WHERE (pid=? OR cid = ?) AND complete = true ORDER BY date DESC";
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sqlUsers)) {
            pstmt.setInt(1, id);
            pstmt.setInt(2, id);

            ResultSet rs  = pstmt.executeQuery();
            if (rs.next()) {
                java.util.Date sqlDate = new java.util.Date(rs.getDate("date").getTime());
                java.util.Date today = new java.util.Date();
                LocalDate date = LocalDate.now();

            }
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        }
    }

    public Calendar findLast () {
        //finds earliest time that matches both users
        Calendar calendar = Calendar.getInstance();
        return calendar;
    }

    public void addAvailability(int id, String date) {
        String sqlUsers = "INSERT INTO schedule(user,date) VALUES(?,?)";
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sqlUsers)) {
            System.out.println(date);
            pstmt.setInt(1, id);
            pstmt.setDate(2, java.sql.Date.valueOf(date));
            pstmt.executeUpdate();
         } catch (Exception e) {
            System.out.println(e.getMessage());
        }
    }

    public String[] findAvailability(int id) {
        Date date;
        String sql = "SELECT date FROM schedule WHERE user = ?";
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sql)) {
            pstmt.setInt(1, id);
            ResultSet rs  = pstmt.executeQuery();
            int size = 0;
            while (rs.next()) {
                size++;
            }
            rs  = pstmt.executeQuery();
            schedule = new String[size];
            int c = 0;
            while (rs.next()) {
                date = rs.getDate("date");
                DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
                String text = df.format(date);
                schedule[c] = text;
                c++;
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        return schedule;
    }

    public void removeAvailability(int id, String date) {
        //finds earliest time that matches both users
        SimpleDateFormat formatter=new SimpleDateFormat("yyyy-MM-dd");
        String sqlUsers = "INSERT INTO schedule(user,date) VALUES(?,?)";
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sqlUsers)) {
            java.util.Date utilDate = formatter.parse(date);
            java.sql.Date sqlDate = new java.sql.Date(utilDate.getTime());
            pstmt.setInt(1, id);
            pstmt.setDate(2, sqlDate);
            pstmt.executeUpdate();
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
    }


    public void addApointment(int id, int pid, int cid, Calendar date, Time sTime, Time eTime) {
        //finds earliest time that matches both users
        SimpleDateFormat formatter1=new SimpleDateFormat("yyyy-MM-dd HH:mm");
        String sqlUsers = "INSERT INTO users(id,pid,cid,date,timeStart,timeEnd) VALUES(?,?,?,?,?,?)";
        try (Connection conn = this.connect();
             PreparedStatement pstmt = conn.prepareStatement(sqlUsers)) {
            pstmt.setInt(1, id);
            pstmt.setInt(2, pid);
            pstmt.setInt(3, cid);
            pstmt.setDate(4, new java.sql.Date(date.getTimeInMillis()));
            pstmt.setTime(5, sTime);
            pstmt.setTime(6, eTime);
            pstmt.executeUpdate();
        } catch (SQLException e) {
            System.out.println(e.getMessage());
        }
    }
}
