import com.sun.jdi.event.MonitorWaitedEvent;

import java.io.*;
import java.net.Socket;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

public class ServerThread extends Thread {

    String name;

    Socket socket;
    InputStream is;
    InputStreamReader isr;
    BufferedReader br;
    int userID;
    PrintWriter out;

    boolean running = true;

    ServerThread(Socket socket) {
        try {
            //Set client Socket
            this.socket = socket;
            //Create client InputStream/readers
            is = socket.getInputStream();
            isr = new InputStreamReader(is);
            br = new BufferedReader(isr);
            out = new PrintWriter(socket.getOutputStream(),true);

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void run() {
        System.out.println("New host connected");
        //initResponse(); //will be used to ensure client is connected later, make sure to read the initial response on the client side when enabling this
        waitState();
    }

    public void initResponse () {
        out.println("CONNECTED");
    }

    public void waitState () {

        try {
            String state = "WAITING";
            while (!Objects.equals(state, "EXIT") && state != null) {
                //check(); check for new requests
                String rl = nextLine();
                System.out.println("Current state is " + rl);
                state = stateMachine(rl);
            }
            socket.close();
        } catch (Exception e) {
            System.out.println("exception in waitState");
        }
        System.out.println("Client disconnected");
    }

    //Reads the next line in the bufferedreader
    public String nextLine () {
        try {
            String s = br.readLine();
            if (s == null) {
                s = "NULL";
            }
            return s;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return "ERROR";
    }

    //FSM
    public String stateMachine (String s) {
        switch(s) {
            case "LOGIN":
                System.out.println("Login attempt made");
                login();
                break;
            case "REGISTER":
                System.out.println("Register attempt made");
                register();
                break;
            case "COMPARE":
                System.out.println("Compare attempt made");
                break;
            case "APPOINTMENT":
                appointment();
                break;
            case "APPOINTMENTCHECK":
                checkSchedule();
                break;
            case "PRIVACY":
                privacy();
                break;
            case "READY":
                ready();
                break;

            case "NULL":
                return "EXIT";
                default:
        }
        return s;
    }

    public void ready() {
        String s = nextLine();
        boolean b = false;
        if (s.equals("TRUE")) {
            b = true;
        }
        Database db = new Database();
        db.changeReady(userID,b);
        int other = db.checkOther(userID);
        if (db.checkReady(userID) && db.checkReady(other)) {
            System.out.println("Both ready");
            response("NEGOTIATE");
            boolean [] s1 = CompareLogic.convertToBoolean(db.checkPrivacy(userID));
            boolean [] s2 = CompareLogic.convertToBoolean(db.checkPrivacy(other));
            boolean [] s3 = CompareLogic.compareUsers(s1,s2);
            db.changePrivacy(userID, CompareLogic.convertToString(s3));
            response(db.checkPrivacy(userID));
        }
        else  {
            response("SUCCESS");
        }
    }

    public void appointment() {
        String date = nextLine();
        Database db = new Database();
        db.addAvailability(userID,date);
        response("SUCCESS");
    }

    public  void checkSchedule() {
        Database db = new Database();
        response("START");
        String schedule[] = db.findAvailability(userID);
        for (int i = 0; i < schedule.length; i++) {
            response(schedule[i]);
        }
        response("END");
    }


    public boolean register() {
        Database db = new Database();
        String user = nextLine();
        String passw = nextLine();
        String email = nextLine();
        String role = nextLine();
        if (db.addUser(user, passw, email, role)) {
            response("REGISTERTRUE");
            return true;
        }
        response("REGISTERFALSE");
        return false;
    }


    public boolean login () {
        String user = nextLine();
        String passw = nextLine();
        Database db = new Database();
        if (db.checkCredential(user, passw)) {
            response("LOGINTRUE");
            userID = db.existingUser(user);
            response(userID);
            System.out.println(db.checkRole(user));
            response(db.checkRole(user));
            db.existingUser(name);
            name = user;
            response(db.checkPrivacy(userID));
            if (db.checkReady(userID)) {
                response("TRUE");
            } else {
                response("FALSE");
            }

            return true;
        }
        response("LOGINFALSE");
        return false;
    }


    public void privacy () {
        String privacy = nextLine();
        Database db = new Database();
        db.changePrivacy(userID, privacy);
        int other = db.checkOther(userID);
        if (db.checkReady(userID) && db.checkReady(other)) {
            System.out.println("Both ready");
            response("NEGOTIATE");
            boolean [] s1 = CompareLogic.convertToBoolean(db.checkPrivacy(userID));
            boolean [] s2 = CompareLogic.convertToBoolean(db.checkPrivacy(other));
            boolean [] s3 = CompareLogic.compareUsers(s1,s2);
            db.changePrivacy(userID, CompareLogic.convertToString(s3));
            response(db.checkPrivacy(userID));
        }
        else  {
            response("SUCCESS");
        }    }


    public void response (String s) {
        System.out.println("Sending " + s + " back to client");
        out.println(s);
    }
    public void response (int i) {
        System.out.println("Sending " + i + " back to client");
        out.println(i);
    }

    public void response (boolean b) {
        System.out.println("Sending " + b + " back to client");
        out.println(b);
    }
}


